#include <stdio.h>
#include <iostream>
#include <signal.h>

/**
 * Optris interface
 */
#include "Imager.h"
#include "ImagerUVC.h"

/**
 * Visualization
 */
#include "Obvious2D.h"
#include "ImageBuilder.h"

using namespace std;
using namespace optris;

#define WIDTH 960
#define HEIGHT 600

obvious::Obvious2D _viewer(WIDTH, HEIGHT, "Optris Imager example");
ImageBuilder _iBuilder;

unsigned char* _bufferShow = NULL;
bool _run                  = true;

void measurePoint(unsigned short* image, unsigned int w, unsigned int h, unsigned int x, unsigned int y)
{
	char text[20];
	_iBuilder.drawCrosshair(_bufferShow, x, y);
	sprintf(text, "%2.1f", 0.1f*((float)(image[x + y*w] - 1000)));
	x -= w/20;
	y += h/20;
	_viewer.addText(text, WIDTH * x/w, HEIGHT - HEIGHT * y/h);
}

void cbOnFrame(unsigned short* image, unsigned int w, unsigned int h)
{
  _iBuilder.convertTemperatureToPalette(image, _bufferShow, eIron);
  _viewer.clearText();
  measurePoint(image, w, h, w/2,   h/2  );
  measurePoint(image, w, h, w/4,   h/4  );
  measurePoint(image, w, h, w/4*3, h/4  );
  measurePoint(image, w, h, w/4,   h/4*3);
  measurePoint(image, w, h, w/4*3, h/4*3);
  _viewer.draw(_bufferShow, _iBuilder.getStride(), h, 3);
}

void terminate (int param)
{
	cout << "Signal raised. Initiate program termination..." << endl;
	_run = false;
}

int main (int argc, char* argv[])
{
	if(argc!=2)
	{
		cout << "usage: " << argv[0] << " <xml configuration file>" << endl;
		return -1;
	}

	/**
	 * ensure proper termination
	 */
	signal (SIGTERM,terminate);
	signal (SIGINT,terminate);

	/**
	 * Initialize UVC interface and determine serial number of Optris imager.
	 * The UVC path is automatically set.
	 */
	ImagerUVC uvc;
	unsigned long serial = uvc.FindFirstDevice();
	if(serial==0)
	{
		cout << "Error finding imager device" << endl;
		return -1;
	}

	if(uvc.OpenDevice()!=0)
	{
		cout << "Error opening UVC device: " << uvc.GetPath() << endl;
		return -1;
	}

	/**
	 * Initialize Optris image processing chain
	 */
	Imager im(serial, uvc.GetWidth(), uvc.GetHeight(), uvc.GetFrequency(), argv[1]);
	im.setAutoflagControl(true);

	uvc.Start();

	_iBuilder.setSize(im.getWidth(), im.getHeight());

	unsigned char* bufferRaw = new unsigned char[uvc.GetWidth() * uvc.GetHeight() * 2];
	_bufferShow              = new unsigned char[_iBuilder.getStride() * im.getHeight() * 3];
	im.setFrameCallback(cbOnFrame);

	//_iBuilder.setTemperatureRange(25.0f, 40.0f);
	_iBuilder.setDynamicScaling(true);

	/**
	 * Enter endless loop in order to pass raw data to Optris image processing library.
	 * Processed data are supported by the frame callback function.
	 */
	while(_run)
	{
		uvc.GetFrame(bufferRaw);
		im.process(bufferRaw);
		uvc.ReleaseFrame();
	}

	uvc.Stop();
	uvc.CloseDevice();

	delete [] bufferRaw;
	delete [] _bufferShow;

	cout << "Exiting application" << endl;
}
