#pragma once

#include "OpUtils.h"
#include "../ImagerDefs.h"

struct Range
{
	float Min, Max;
	Range(void)	{ Min = Max = 0; }
	Range(float mn, float mx) { Min = mn; Max = mx; }
	Range(Range *range) { Min = range->Min; Max = range->Max; }
	bool operator==(Range* range) {	return (Min == range->Min) && (Max == range->Max); } //   '=='
	bool operator!=(Range* range) {	return (Min != range->Min) || (Max != range->Max); } //   '!='
};

	class MainSection
	{
	public:
		WORD TecA, TecB, TecC, TecD;
		float TempTEC_Step;
		float TempTEC_AutoOffset;
		float TempTECOffset;
		float TempTECGain;
		float 	TempFlagOffset;
		float 	TempChipFactor;
		float 	TempChipOffset;
		float 	TempBoxOffset;
		float 	TempProcOffset;
		float 	TempHouseOffset;
		float 	PifInOffset;
		float 	PifInGain;
		float 	PifOutOffset;
		float 	PifOutGain;
		WORD AddedOffset;
		float 	BindTempChipToTempFlagOffset;
		WORD FlagCycles;
		WORD FlagCycles_Ignore;

		// ctor:
		MainSection(void);
	};


	class TempTableParams
	{
	public:
		double T1, T3;		
		short E1, E3;
		double Lambda; // the central wave length of the Planck function
		// c'tor:
		TempTableParams()
		{
			T1 = T3 = 0.0;
			E1 = E3 = 0;
			Lambda = 0.0;	// the default value of 0.0 notifys the application that these parameters aren't present
							// in this case the application must try to open the temp-table file "Kennlinie-xxxx.prn"
							// that is delivered by MathCad		
                }
		TempTableParams(double t1, double t3, short e1, short e3, double lambda)
                {	T1 = t1; T3 = t3; E1 = e1; E3 = e3;	Lambda = lambda; }
                TempTableParams(WORD t1, WORD t3, short e1, short e3, long lambda)
		{	
			T1 = 0.1 * (double(t1) - 1000.0); 
			T3 = 0.1 * (double(t3) - 1000.0); 
			E1 = e1; 
			E3 = e3;	
			Lambda = 1E-6 * lambda; 
                }
		TempTableParams(TempTableParams *TableParams)
		{
			T1 = TableParams->T1;
			T3 = TableParams->T3;
			E1 = TableParams->E1;
			E3 = TableParams->E3;
			Lambda = TableParams->Lambda;
                }
	};

#define DEFAULTOPTICSCOUNT 1
#define DEFAULTTEMPRANGESCOUNT 3

	class TemperatureRange
	{
	public:
                unsigned short Fid, Skim;
		float TempLo, TempHi;
		TempTableParams *TableParams;
		// c'tor:
		TemperatureRange(void) 
		{
			TableParams = new TempTableParams();
                }
		// d'tor:
		~TemperatureRange(void) 
		{
			SAFE_RELEASE_OBJ(TableParams);
		}
	};
	typedef TemperatureRange *PTemperatureRange;

	class OpticsDef
	{
	public:
		int FOV;
		int SerNo;
		double RadialDistortion;
		PTemperatureRange* TempRanges;
		int TempRangeCount;

		// ctor:
		OpticsDef(void) 
		{
			FOV = 0;
			SerNo = 0;
			TempRangeCount = DEFAULTTEMPRANGESCOUNT;
			TempRanges = new PTemperatureRange[TempRangeCount];
			for(int i =0; i<TempRangeCount; i++)
			{
				TempRanges[i] = new TemperatureRange();
				TempRanges[i]->Fid  = 0;
				TempRanges[i]->Skim = 0;
			}
			TempRanges[0]->TempLo = -20.0;
			TempRanges[0]->TempHi = 100.0;
			TempRanges[1]->TempLo =   0.0;
			TempRanges[1]->TempHi = 250.0;
			TempRanges[2]->TempLo = 150.0;
			TempRanges[2]->TempHi = 900.0;
			RadialDistortion = -1;
                }
		// ctor:
		~OpticsDef(void) 
		{
			for(int i =0; i<TempRangeCount; i++)
				SAFE_RELEASE_OBJ(TempRanges[i]);
			SAFE_RELEASE(TempRanges);
                }
	};
	typedef OpticsDef *POpticsDef;

	class TemperatureSection
	{
	public:
		POpticsDef* Optics;
		int OpticsCount;
		// ctor:
		TemperatureSection(void) 
		{
			OpticsCount = DEFAULTOPTICSCOUNT;
			Optics = new POpticsDef[OpticsCount];
			for(int i =0; i<OpticsCount; i++)
			{
				Optics[i] = new OpticsDef();
			}
			Optics[0]->FOV  = 30;
			Optics[0]->SerNo = 0;
                }
		~TemperatureSection(void) 
		{
			for(int i =0; i<OpticsCount; i++)
				SAFE_RELEASE_OBJ(Optics[i]);
			SAFE_RELEASE(Optics);
                }
	};

class ConfigCali
{
public:
	MainSection *Main;
	TemperatureSection *Temperature;
	ConfigCali(void);
	~ConfigCali(void);
        long Load(wchar_t *Dir, int SerialNumber);
	int GetOpticsCount(void);
	int GetOpticsIndex(int FOV);
	int GetOpticsFOV(int Index);
	int GetTempRangeCount(int FOV);
	int GetTempRangeIndex(int FOV, float TempRangeMin, float TempRangeMax);
	int GetTempRangeIndex(int FOV, Range *TempRange);
	Range* GetTempRange(int FOV, int Index);
	Range* GetTempRange_OverAll(int FOV);
	Range* GetTempRange_OverAll(void);
};
