#pragma once

#ifdef WIN32
#include <windows.h>
#endif
#include <pthread.h>
#include "../ImagerTypes.h"
#include "../ImagerDefs.h"
#include "../ImagerHID/ImagerHID.h"
#include "../MyDefs.h"
#include "OpBase.h"
#include "Cali.h"

#define Offset2COUNT 10

// ---  Class ImageProcessing -----------------------------------
class ImageProcessing;
typedef ImageProcessing *PImageProcessing;
class ImageProcessing
{
public:
	ImageProcessing(void);
	~ImageProcessing(void);

	int OperationsCount;
	POpBase *Operations;

// --- General public functions ----------------------------------
	HRESULT Create(WORD ImgIndex, ImagerHID* hid, ImgProcConfig *imgProcConfig, FrameConfig *frameConfig, void **buffer, int imgProcSettings);
	HRESULT Init(void);
	HRESULT GetConfig(IPDataType datatype, PFrameConfig config);
	HRESULT GetCurrentBuffer(IPDataType datatype, long bufferSize, PVOID buffer);
	HRESULT GetCurrentMetadata(IPDataType datatype, PFrameMetadata metadata);
	HRESULT NewFrame(PNewFrameBuffer Buffer);
	HRESULT RepeatFrame(void);
	void OnTimer(void); 
protected:
	void Init(IPOperation index, PProcessingBuffer *buf);
	void Release(void);
        HRESULT ReadCali(void);
	HRESULT ProcessFrame(void);
        IPOperation CreateOp(IPOperation opPrev, IPOperation opNew, WCHAR *Name);
	fpOnSnapshot m_fpOnSnapshot;
//	HANDLE TimerHandle;
        pthread_t TimerHandle;
	DWORD MainTimerNext, MainTimerDivider;

	void MainTimer100ms(void);
	void MainTimer200ms(void);
	void MainTimer500ms(void);
	void MainTimer1000ms(void);
	void MainTimer5000ms(void);


#ifdef WIN32
    CCritSec   csShared;		// Protects shared data.
#endif
    typedef ThreadMutex MUTEX_TYPE;
    MUTEX_TYPE g_mtx;

// --- General interface functions ----------------------------------
public:
	STDMETHODIMP GetCaliOpticsCount(int *pOpticsCount);
	STDMETHODIMP GetCaliOpticsIndex(int FOV, int *pOpticsIndex);
	STDMETHODIMP GetCaliOpticsFOV(int Index, int *pFOV);
	STDMETHODIMP GetCaliTempRangeCount(int FOV, int *pTempRangeCount);
	STDMETHODIMP GetCaliTempRangeIndex(int FOV, float TempRangeMin, float TempRangeMax, int *pTempRangeIndex);
	STDMETHODIMP GetCaliTempRangeMin(int FOV, int Index, float *pTempRangeMin);
	STDMETHODIMP GetCaliTempRangeMax(int FOV, int Index, float *pTempRangeMax);
	STDMETHODIMP SetOpticsAndTemprange(int FOV, float TempRangeMin, float TempRangeMax);
	STDMETHODIMP InitCali(WORD *HWRev, WORD *FWRev, ULONG *PifSerial, bool *LimitFramerate);
	STDMETHODIMP SetFlag(TFlagState Val);
	STDMETHODIMP StartFlag(void);
	STDMETHODIMP SetPifOut(WORD Voltage);
	STDMETHODIMP ConfigPIF(PIFConfigStruct *pConfig);
	STDMETHODIMP GetMissingCaliFilename(WCHAR *pFilename, int Len);

// --- General Variables --------------------------------------------
protected:
	PProcessingBuffer SourceBuffer;
	ImgProcConfig cfg;
	GlobalVars Global;
	ConfigCali *Cali;

	long long FrameCounter0;

	IPOperation OpIndices[IPDataTypeCOUNT];
	IPOperation OpSourceIR;
};

