#pragma once

#ifdef WIN32
#include <assert.h>
#include <windows.h>

#define ASSERT(_X_)  assert(_X_)

#endif

#include <wchar.h>
#include "OperationTypes.h"
#include "OpUtils.h"
#include "Cali.h"
#include "../MyDefs.h"

class OpBase;
typedef OpBase * POpBase;
typedef POpBase * PPOpBase;
class OpBase
{
public:
        OpBase(OpBase *prevOp, IPOperation op, wchar_t *name, PGlobalVars global);
	virtual ~OpBase(void);
        PProcessingBuffer InitProcessingBuffer(PBufferConfig config, IPDataType DataType);
        virtual long Process(void);
        virtual long RemoveEvents(void);
        virtual long Init(POpBase prevOp) = 0;
	POpBase GetOpObject(IPOperation op, bool backward, bool forward);
//        int TestMessageBox(char* caption, char* format, ...);

	PProcessingBuffer Buffer;
	float SharedTimeSum;

	int Count;
	POpBase Prev, *Next;
	POpBase AlsoInit; // An item that must also be initialized
	POpBase AlsoPrev; // An item from that comes an additional initialization
	RECT ClipRect;
	bool Clipping;
	IPOperation IPOp;
	fpOnFrame OnFrame;
	fpOnFrameInit OnFrameInit;
	int Width;
	int Height;
	int Size;

protected:
        virtual long Init(PProcessingBuffer *Out);
        virtual long Reinit(void);
        virtual long Processing(void) = 0;
	void Add(POpBase Op);
	virtual void SetClipRect(RECT clipRect, bool SetThis, bool SetPrevious);
	void ReleaseBuffers(void);
	void ReleaseBuffer(PProcessingBuffer *buffer);

#ifdef WIN32
    CCritSec   csShared;		// Protects shared data.
#endif
        typedef ThreadMutex MUTEX_TYPE;
        MUTEX_TYPE g_mtx;

	PGlobalVars Global;
        wchar_t *Name;

	bool OwnerOfBuffer;
	bool IsProcessing;
        long long PerfFrequency;

// --- General interface functions ----------------------------------
        long SetConfigDirectory(wchar_t* pDirectory);
        long SetBool(bool Val, bool *pParam);
        long GetBool(bool *pVal, bool *pParam);
        long SetDouble(double Val, double *pParam);
        long GetDouble(double *pVal, double *pParam);
        long SetFloat(float Val, float *pParam);
        long GetFloat(float *pVal, float *pParam);
        long SetINT(int Val, int *pParam);
        long GetINT(int *pVal, int *pParam);
        long SetWORD(WORD Val, WORD *pParam);
        long GetWORD(WORD *pVal, WORD *pParam);
        long SetDWORD(DWORD Val, DWORD *pParam);
        long GetDWORD(DWORD *pVal, DWORD *pParam);
        long SetBYTE(BYTE Val, BYTE *pParam);
        long GetBYTE(BYTE *pVal, BYTE *pParam);
        long SetPOINT(POINT Val, POINT *pParam);
        long GetPOINT(POINT *pVal, POINT *pParam);
        long GetTestData(DWORD** pVal, DWORD *pSize);

// --- Interface functions ----------------------------------------------------
public:
        long GetName(wchar_t *pName, int MaxLen);
        long GetResolution(SIZE *pVal);
        long GetFramerate(double *pVal);
};

