#ifndef IMAGERHID_H
#define IMAGERHID_H

#include "../ImagerTypes.h"
#include "../ImagerDefs.h"
#include "HIDComm/HIDComm.h"
#include "HIDDevice/HIDDevice.h"

class ImagerHID;
typedef ImagerHID *PImagerHID;

#ifndef max
#define max(a,b)    (((a)>(b))?(a):(b))
#endif

#ifndef min
#define min(a,b)    (((a)<(b))?(a):(b))
#endif

class ImagerHID
{
public:
    float                   TempTecMax;
    float                   TempTecMin;
    float                   TempBoxOffset;
    float                   TempChipFactor;
    float                   TempChipOffset;
    float                   TempFlagOffset;
    float                   TempTecOffset;
    float                   TempTecGain;
    float                   PifInOffset;
    float                   PifInGain;
    float                   PifOutOffset;
    float                   PifOutGain;

    ImagerHID();
    ~ImagerHID();
    int Init(unsigned long VidImager, unsigned long PidImager);     // HRESULT --> int
    int Init(unsigned long VidImager, unsigned long PidImager, unsigned long Sernr);
    int SetHIDLog(int Loglevel, WCHAR *Filename);
    bool ImagerGetDeviceInfo(void);
    unsigned long InitPif(void);
    int GetHIDBuffer(unsigned char *buffer, int len);
    unsigned long GetSerialNumber(void);
    unsigned short GetFirmwareRev(void);
    unsigned short GetHardwareRev(void);
    void GetPifIn(WORD *pVoltage);
    void SetPifOut(WORD Voltage);
    void SetTecEnable(bool on);
    void GetTecEnable(bool *pVal);
    void SetTempTec(float Temp);
    void GetTempTec(float *pVal);
    void GetFlag(TFlagState *pVal);
    void SetFlag(TFlagState Val);
    void SetFlagCycle(unsigned short CycleCount);
    void SetTecA(unsigned short Val);
    void GetTecA(unsigned short *pVal);
    void SetTecB(unsigned short Val);
    void GetTecB(unsigned short *pVal);
    void SetTecC(unsigned short Val);
    void GetTecC(unsigned short *pVal);
    void SetTecD(unsigned short Val);
    void GetTecD(unsigned short *pVal);
    void SetSkim(unsigned short voltage);
    void SetSkim_WaitForFlag(unsigned short voltage);
    void GetSkim(unsigned short *pVal);
    void SetSkim_Adjust(unsigned short voltage);
    void GetSkim_Adjust(unsigned short *pVal);
    void SetFid(unsigned short voltage);
    void SetFid_WaitForFlag(unsigned short voltage);
    void GetFid(unsigned short *pVal);
    void SetFid_Adjust(unsigned short voltage);
    void GetFid_Adjust(unsigned short *pVal);
    void SetBiasEnable(bool on);
    void GetBiasEnable(bool *pVal);
    void SetPowerEnable(bool on);
    void GetPowerEnable(bool *on);
    bool GetShortImagerCaps(void);

    TPifMode GetPIFInMode(void);
    void SetPIFInMode(TPifMode Val);
    TPifMode GetPIFInDigitalMode(void);
    void SetPIFInDigitalMode(TPifMode Val);
    void GetPIFOutMode(TPifMode *pVal);
    void SetPIFOutMode(TPifMode Val);
    unsigned short GetPIFInFlagThreshold(void);
    void SetPIFInFlagThreshold(unsigned short val);
    bool GetPIFInFlagOpenIfLower(void);
    void SetPIFInFlagOpenIfLower(bool val);
    void SetPIFOutFlagOpen(unsigned short value);
    void GetPIFOutFlagOpen(unsigned short *pVal);
    void SetPIFOutFlagClosed(unsigned short value);
    void GetPIFOutFlagClosed(unsigned short *pVal);
    void SetPIFOutFlagMoving(unsigned short value);
    void GetPIFOutFlagMoving(unsigned short *pVal);
    bool GetPIFInDigitalFlagLowActive(void);
    void SetPIFInDigitalFlagLowActive(bool val);

    unsigned short GetPIFInThreshold(void);
    void SetPIFInThreshold(unsigned short val);
    bool GetPIFInOpenIfLower(void);
    void SetPIFInOpenIfLower(bool val);
    bool GetPIFInDigitalLowActive(void);
    void SetPIFInDigitalLowActive(bool val);

private:
    HIDDevice               HIDDev;
    bool                    WaitGetAllValues;
    bool                    TecEnable;
    bool                    m_bFlag;
    bool                    m_bPwrCtl;
    unsigned short	    m_uiTec;
    unsigned short	    uiTecA;
    unsigned short	    uiTecB;
    unsigned short	    uiTecC;
    unsigned short	    uiTecD;
    unsigned short	    m_uiSkim;
    unsigned short	    m_uiFid;
    unsigned short	    uiPifIn;
    unsigned short	    uiPifOut;
    bool                    m_bBias;
    bool                    m_PifInDigital;
    bool                    m_IsOpen;
    unsigned long           m_uiVid;
    unsigned long           m_uiPid;
    unsigned long           m_uiSernr;  //change format from _int64 to unsigned long
    wchar_t*                m_strManufacturer;
    wchar_t*                m_strProduct;
    unsigned long           m_uiVersion;
    unsigned long           m_uiCount;
    TFlagState              m_FlagState;
    float                   m_TempProc;
    float                   m_TempChip;
    float                   m_TempFlag;
    float                   m_TempBox;
    float                   m_TempHouse;
    float                   m_TempTec;
    unsigned short          m_FWCypVers;
    unsigned short          m_FWMspVers;
    unsigned short          m_HardwareRev;
    DEVICE_DESCRIPTION*     pdevdesc;
    TEMPERATURE*            ptemperature;
    PI_HKD*                 pHkd;
    unsigned short          mp_Interleave;
    unsigned short	    m_FrameCounter;
    TPifMode                m_PifInMode;
    TPifMode                m_PifInDigitalMode;
    TPifMode                m_PifOutMode;
    unsigned short	    m_PifInFlagThreshold;
    bool		    m_PifInFlagOpenIfLower;
    unsigned short	    m_PifInThreshold;
    bool		    m_PifInOpenIfLower;
    unsigned short	    m_PifOutFlagOpen;
    unsigned short	    m_PifOutFlagClosed;
    unsigned short	    m_PifOutFlagMoving;
    bool		    m_PifInDigitalFlagLowActive;
    bool		    m_PifInDigitalLowActive;

    void OpenImagerDevice(void);
    void clearglobal(void);
    void sleep_ms(unsigned int mseconds);

};

#endif // IMAGERHID_H
