#pragma once

#include <string.h>
#include "OpBase.h"
#include "OpDefs.h"
#include <fstream>
using namespace std;


//{7A9AAE14-E7AE-485f-B73A-946F94C061AB}
static const GUID guid_164_120_100 = { 0x7A9AAE14, 0xE7AE, 0x485f, { 0xB7, 0x3A, 0x94, 0x6F, 0x94, 0xC0, 0x61, 0xAB } };
//{7A9AAE14-E7AE-485f-B73A-946F94C06100}
static const GUID guid_160_121_120 = { 0x7A9AAE14, 0xE7AE, 0x485f, { 0xB7, 0x3A, 0x94, 0x6F, 0x94, 0xC0, 0x61, 0x00 } };
//{BC46143A-F587-442b-B2A5-85CEF8CACA87}
static const GUID guid_760_480_32 = { 0xBC46143A, 0xF587, 0x442b, { 0xB2, 0xA5, 0x85, 0xCE, 0xF8, 0xCA, 0xCA, 0x87 } };
//{AEF45D39-5BD2-4918-AF00-B3EE9542B1CE}
static const GUID guid_384_288_66 = { 0xAEF45D39, 0x5BD2, 0x4918, { 0xAF, 0x00, 0xB3, 0xEE, 0x95, 0x42, 0xB1, 0xCE } };

#define HIDBuferLen 256

#pragma pack(push)
#pragma pack(1)

typedef struct _SOURCE_METADATA
{
	unsigned short TempProc; 		//0+1
	unsigned short TempChip; 		//2+3
	unsigned short TempFlag; 		//4+5
	unsigned char I2C_exp_front; 	//6
	unsigned short TempBox;			//7+8
	unsigned short PifIn;			//9+10
	unsigned char FlagState;		//11
	unsigned char Usb_Speed;		//12
	unsigned short HouseTemp;		//13+14
	unsigned short FwCypVers;		//15+16
	unsigned short FwMspVers;		//17+18
	unsigned short Handling;		//19+20
	unsigned long FrameCounter;		//21-24
	unsigned short CountReadRequest;//25+26
	unsigned short CountReadResponse;//27+28
	unsigned char ReadCommand;		 //29
} SOURCE_METADATA;

#pragma pack(pop)


class OpSource : public OpBase
{
public:
	OpSource(OpBase *prevOp, IPOperation op, WCHAR *name, PGlobalVars global, ConfigCali *cali, IPDeviceType deviceType);
	~OpSource(void);
	HRESULT Process();

protected:
	char *GetKey(char * str, char * Key, char delimiter);
	bool GetValue(int *Value, char ** str);
	bool GetBlock(SubframeBlock *block, char ** str);
	char *Trim(char * str);
	HRESULT Init(POpBase prevOp);
	HRESULT Processing(void);
	HRESULT PrepareArranging(int BufferSize);
	TChipModel DetectChipModel(PVideoConfig pVC);
	GUID GetGuid(PVideoConfig pVC);
	void CheckFrameTimeout(void);
	void ReadSourceMetadata(SOURCE_METADATA *pSMD);
	GUID ParseGuid(char *guid_str);
	HRESULT ReleaseFormatDefinition(FormatDef **pformatdef); 
	HRESULT DefaultFormatDefinition(FormatDef **pformatdef, int Width, int Height); 
	HRESULT LoadFormatFromFile(FormatDef **pformatdef, IPFormatParseType ParseType, GUID Guid, VideoConfig *pVCin, IPDeviceType DevType);
	HRESULT TranslateVideoConfig(PVideoConfig pVCin, PVideoConfig pVCout, IPDeviceType DevType);

protected:
	unsigned char m_HIDBuffer[HIDBuferLen];
	TChipModel ChipModel;
	bool MaskBit14, MaskBit15;
	DWORD m_LastCheckFrameTimeout;
	WORD m_FrameTimeoutCounter;
	WORD SubFrameCounter;
	IPDeviceType DeviceType;
	FormatDef *pFormatDef;
	ConfigCali *Cali;

#ifdef _DEBUG
	int T;
	long cnt, CNT, CNTHW;
#endif

// --- Interface functions ----------------------------------------------------
public:
	STDMETHODIMP GetVideoConfig(PVideoConfig pVC_in, PVideoConfig pVC_out, int Channels, int *pRead);
};
