#pragma once

#define minmax(x, a, b) (((x) < (b))   ?   (((x) > (a)) ? (x) : (a))   :   (b))
#define SAFE_RELEASE(pointer) { if(pointer != NULL)	{ delete [] pointer; pointer = NULL; } }
#define SAFE_RELEASE_OBJ(pointer) { if(pointer != NULL)	{ delete pointer; pointer = NULL; } }
#define CHECK_POINTER(p) {if((p)==NULL) return (-1);}
#define CHECK_RETURN(_hr_) {if((_hr_) != 0) return (_hr_);}

#ifdef WIN32
#include <windows.h>

// wrapper for whatever critical section we have
class CCritSec 
{
    // make copy constructor and assignment operator inaccessible
    CCritSec(const CCritSec &refCritSec);
    CCritSec &operator=(const CCritSec &refCritSec);
    CRITICAL_SECTION m_CritSec;

public:
    CCritSec() { InitializeCriticalSection(&m_CritSec); }
    ~CCritSec() { DeleteCriticalSection(&m_CritSec);  }
    void Lock() { EnterCriticalSection(&m_CritSec); }
    void Unlock() { LeaveCriticalSection(&m_CritSec); }
};

// locks a critical section, and unlocks it automatically
// when the lock goes out of scope
class CAutoLock 
{
    // make copy constructor and assignment operator inaccessible
    CAutoLock(const CAutoLock &refAutoLock);
    CAutoLock &operator=(const CAutoLock &refAutoLock);
protected:
    CCritSec * m_pLock;

public:
    CAutoLock(CCritSec * plock)
    {
        m_pLock = plock;
        m_pLock->Lock();
    }
    ~CAutoLock() 
	{
        m_pLock->Unlock();
    }
};
#define AUTOLOCK CAutoLock lock(&csShared);
#define AUTOLOCK_OBJ(_obj) CAutoLock lockObj(&((_obj)->csShared));

#endif

#include <pthread.h>

struct ThreadMutex
{
        ThreadMutex()
        {
                pthread_mutex_init(&mtx,NULL);
        }

        ~ThreadMutex()
        {
                pthread_mutex_destroy( &mtx );
        }

        inline void lock()
        {
                pthread_mutex_lock( &mtx );
        }

        inline void unlock()
        {
                pthread_mutex_unlock( &mtx );
        }
        pthread_mutex_t mtx;
};

template<class T>
class CAutoGuard
{
public:
        CAutoGuard(T &mtx) : m_mtx(mtx)
        {
                m_mtx.lock();
        }
        ~CAutoGuard()
        {
                m_mtx.unlock();
        }
protected:
        T &m_mtx;
};

#define AUTO_GUARD(guard_tmp_var, MUTEX_TYPE, mtx) CAutoGuard<MUTEX_TYPE> guard_tmp_var(mtx)
#define AUTOLOCK AUTO_GUARD(gd, MUTEX_TYPE, g_mtx );
