#ifndef HIDDEVICE_H
#define HIDDEVICE_H

#include "../HIDComm/HIDComm.h"
#include "../../ImagerDefs.h"

typedef struct DEVICE_HKD{
    unsigned char	HW_Model;
    unsigned char	HW_Special;
    unsigned char	Reserve_1;
    unsigned char	Reserve_2;
    unsigned long	Serialnumber;
    unsigned short	FW_MspVersion;
    unsigned short	FW_CypVersion;
    unsigned short	FW_DLLMajor;
    unsigned short	FW_DLLMinor;
    unsigned short	FW_DLLBuild;
    unsigned short	FW_DLLRevision;/**/
    unsigned long	OpticsSerNo;
    unsigned long	FocalLength;
    unsigned short	FilterType;
    unsigned short	Reserve_3;
    unsigned short	PixelperLine;
    unsigned short	LinesperFrame;
    unsigned short	FramesperSec;
    unsigned short	PixClock;
    unsigned short	Tint;
    unsigned long	UlisSerial;
    unsigned char	GainControl;
    unsigned char	AduBits;
    unsigned short      AduRangeLow;
    unsigned short	AduRangeHigh;
    unsigned short	AddedOffset;
    unsigned short	VideoAmpl;
    unsigned short	BiasFid;
    unsigned short	BiasSkim;
    unsigned short	BiasFid_Adjust;
    unsigned short	BiasSkim_Adjust;
    unsigned short	TecA;
    unsigned short	TecB;
    unsigned short	TecC;
    unsigned short	TecD;
}PI_HKD;

typedef struct _PIF_EEPROM_Pif
{
        unsigned short Factor;
        short Offset;
        unsigned char ShiftFactor;
} PIF_EEPROM_Pif;
typedef struct _PIF_EEPROM
{
        unsigned char Size;
        unsigned char Version;
        unsigned long SerNo;
        _PIF_EEPROM_Pif PifIn;
        _PIF_EEPROM_Pif PifOut;
        unsigned char Checksum;
} PIF_EEPROM;

#pragma pack(push)
#pragma pack(1)
typedef struct _PIF_EEPROM_Pif_Packed
{
        unsigned short Factor;
        short Offset;
        unsigned char ShiftFactor;
} PIF_EEPROM_Pif_Packed;
typedef struct _PIF_EEPROM_Packed
{
        unsigned char Size;
        unsigned char Version;
        unsigned long SerNo;
        _PIF_EEPROM_Pif_Packed PifIn;
        _PIF_EEPROM_Pif_Packed PifOut;
        unsigned char Checksum;
} PIF_EEPROM_Packed;
#pragma pack(pop)

#ifdef PIINTERFACE_API_IMPORTS
PIINTERFACE_API
#endif

struct DEVICE_SETUP
{
    unsigned int    uiTpoc;
    unsigned int    uiTchip;
    bool            bFlag;
};

struct TEMPERATURE
{
    unsigned short  TChip;
    unsigned short  TFlag;
    unsigned short  TBox;
};

class HIDDevice
{
    HIDComm Hidcomm;
public:
    HIDDevice();
    ~HIDDevice();
    int GetHIDDeviceInfo(struct DEVICE_DESCRIPTION *ptr_devinf);
    bool OpenHIDDevice(struct DEVICE_DESCRIPTION *ptr_devinf);
    int CloseHIDDevice(void);
    int SetLogging(int Loglevel,  WCHAR* Filename);

    int SetTec(unsigned int Value);
    int SetTec_A(unsigned int Value);
    int SetTec_B(unsigned int Value);
    int SetTec_C(unsigned int Value);
    int SetTec_D(unsigned int Value);
    int SetSkimFlex(unsigned int Value, bool WaitForFlag);
    int SetFidFlex(unsigned int Value, bool WaitForFlag);
    int EnableBias(bool On);
    int GetFlag(unsigned char *Open);
    int SetFlag(bool Open);
    int GetTproc(float *ptr_TProc);
    int GetTchip(float *ptr_TChip);
    int GetTFlag(float *Temp);
    int GetTBox(float *Temp);
    int GetTHouse(float *Temp);
    int GetPif( unsigned short* Temp);
    int GetDeviceSetup(struct DEVICE_SETUP *ptr_devsetup);
    int SetPwrCtl(bool On);
    int SetTecCtl(bool On);
    int SetPif(unsigned short* value);
    int SteuerFlagCycle(unsigned short * CycleCount);
    int GetFwCypVers(unsigned short * FwVers);
    int SetPifInMode(unsigned char mode);
    int SetPifOutMode(unsigned char mode);
    int SetPifInDigitalMode(unsigned char mode);
    int SetPifOutFlagOpen( unsigned short* value);
    int SetPifOutFlagClosed( unsigned short* value);
    int SetPifOutFlagMoving( unsigned short* value);
    int SetPifInFlagThreshold(unsigned short val);
    int SetPifInFlagOpenIfLower(bool val);
    int SetPifInDigitalFlagLowActive(bool val);
    int GetPifEEPROM(PIF_EEPROM *PifEeprom);
    bool GetPiHZVersion(void);
    bool GetTempSensorsHighResolution(void);
    int GetHardwareRev(unsigned short *HardwareRev);
    int GetHIDBuf(unsigned char *buffer, int len, unsigned int *BytesRead);

};

#endif // HIDDEVICE_H
