#pragma once
//#include <uuid/uuid.h>
#include "MyDefs.h"
#include "ImagerDefs.h"

// other strings:
#define ConfigDataSubDir L"Imager"

// other filter definitions

#define ENERGYTABMIN (-3000)
#define ENERGYTABMAX (0x4000)
#define ENERGYTABSIZE (ENERGYTABMAX-ENERGYTABMIN)

#define INVALIDPIXEL ((short)0x8000)

#define Minimum_RangeDiff_ADU	100
#define Minimum_RangeDiff_Temp  40

enum TTempPickerMode {tpmMin, tpmMean, tpmMax, tpmCalculation, tpmPIFin };
enum TTempPickerCalculationOperation {tpcoOff, tpcoDiff, tpcoAbsDiff };

struct TempPickerInfoHeader
{
	int size;
	int TempPickerIndex;
	RECT rect;
	int frames;
	TTempPickerMode mode;
	bool polling;
	float LoAlarm;
	float HiAlarm;
	bool EnableAlarmEvent;
	bool EnableAlarmToPIF;
	bool SnapshotOnAlarm;
	short SnapshotOnAlarmDelay;
	short OperandCount;
	short *Operands;
	TTempPickerCalculationOperation Operation;
	bool* matrix;
};

enum TExtColorsMode {ecmOff, ecmColors, ecmPalette, ecmPaletteExt};
#define PALETTES ecmPaletteExt+1

// Temp2Color filter:
// ARM = Auto Range Mode
#define ARM_Manual 1
#define ARM_MinMax 2
#define ARM_1Sigma 3
#define ARM_3Sigma 4
// TPM = Temperature Picker Mode
#define TPM_NONE      0
#define TPM_Single    1
#define TPM_Continous 2


enum TReduceMode {rmOff, rmSkip, rmAvg, rmMin, rmMax, rmAvgExt};
enum TFlagState {fsFlagOpen, fsFlagClose, fsFlagOpening, fsFlagClosing, fsError};
enum TPifMode	{ pifmOff, pifmFlag, pifmRecording, pifmSnapshot, pifmLinescanner, pifmUser=0xffff };
enum TAlarmState {asNoAlarm, asLowAlarm, asHighAlarm};
enum TAutoSkimMode { asmOff, asmOn, asmConditional };
enum TChipModel
{
	UnknownChipModel,
	UL_02_15_2, // Ceramic, 160 x 120
	UL_02_15_1,	// GoldCap, 165 x 120
	Omnivision
};

enum LSOrientation
{
	OrientTopDown,
	OrientBottomUp,
	OrientLeftToRight,
	OrientRightToLeft
};

enum LSMode
{
	LSMcont, // continuous mode
	LSMtrig, // external triggered mode
	LSMself  // self triggered mode
};

struct LSSelfTrigger
{
	bool UseThresholdMin;
	bool UseThresholdMax;
	float ThresholdTempMin;
	float ThresholdTempMax;
	float HoldoffTime;
	RECT Rect;
};

enum IPDataType
{
	ipdtNone,
	ipdtEnergy,				// energy values (ADU)
	ipdtRecording,			// for recording (might be energy or color)
	ipdtTemp,				// temperature
	ipdtColorClass,			// color classes
	ipdtColor,				// colors
        ipdtVisible			// visible channel
};
// must be adapted if new ipdt values are defined:
#define IPDataTypeCOUNT 7

enum TUseRelativeTemps
{
	urtNone    =0x00,
	urtAreasMin=0x01,
	urtAreasMax=0x02,
	urtSpotsMin=0x04,
	urtSpotsMax=0x08,
	urtRangeMin=0x10,
	urtRangeMax=0x20
};

enum TTestPattern {tpNone, tpSquare, tpNoise, tpSquareNoise};

#define ImagerError_FrameTimeout 0x0100
#define ImagerError_FlagTimeout 0x0200
#define ImagerError_FlagTooShort 0x0201

struct ImgProcConfig;
struct VideoConfig;
struct FrameConfig;
struct FrameBuffer;
struct NewFrameBuffer;
struct FrameMetadata;
struct SnapshotBuffer;
typedef ImgProcConfig *PImgProcConfig;
typedef VideoConfig *PVideoConfig;
typedef FrameConfig *PFrameConfig;
typedef FrameBuffer *PFrameBuffer;
typedef NewFrameBuffer *PNewFrameBuffer;
typedef FrameMetadata *PFrameMetadata;
typedef SnapshotBuffer *PSnapshotBuffer;

typedef void (*fpOnNewFrame)(WORD ImgIndex, PNewFrameBuffer Buffer);
typedef void (*fpOnFrame)(WORD ImgIndex, PFrameBuffer Buffer);
typedef void (*fpOnFrameInit)(WORD ImgIndex, PFrameConfig Config);
typedef void (*fpOnFlagState)(WORD ImgIndex, TFlagState flagstate, double LastTime, int LastCount);
typedef void (*fpOnError)(WORD ImgIndex, WORD ImagerError);
typedef int (*fpOnAutoSkim)(WORD ImgIndex, int adjust, bool immediately);

typedef void (*fpOnLinescanningStopped)(WORD ImgIndex, int lines, bool AutoStopped);  // Linescanner image complete
typedef void (*fpOnLinescanningStarted)(WORD ImgIndex, bool AutoStarted);
typedef void (*fpOnLinescanningTime)(WORD ImgIndex, double time);

typedef void (*fpOnWriteDiffImageReady)(WORD ImgIndex);

typedef void (*fpOnFrameNumberReset)(WORD ImgIndex);
typedef void (*fpOnTempRangeChanged)(WORD ImgIndex, unsigned short TempMin, unsigned short TempMax);
typedef void (*fpOnTempPickerEx)(WORD ImgIndex, int len, unsigned short *data);
typedef void (*fpOnSpotMove)(WORD ImgIndex, int x, int y);
typedef void (*fpOnAlarm)(WORD ImgIndex, int Index, TAlarmState State, WORD Temp);
typedef void (*fpOnSnapshot)(WORD ImgIndex, PSnapshotBuffer buffer);
typedef void (*fpOnPIF)(WORD ImgIndex, bool State);


struct ImgProcConfig
{
	int SerialNumber;
        wchar_t ConfigDirectory[MAX_PATH];
        wchar_t CaliDirectory[MAX_PATH];
        wchar_t ApplDirectory[MAX_PATH];
	fpOnFrameInit OnFrameInit[IPDataTypeCOUNT];
	fpOnFrame OnFrame[IPDataTypeCOUNT];
};

#define IP_LiveData					0x0001
#define IP_Recording				0x0002
#define IP_Radiometric				0x0004
#define IP_Linescanning				0x0008
#define IP_HighspeedTemps			0x0010
#define IP_Colors					0x0020
#define IP_VisibleCam				0x0040
#define IP_Playback_FrameMetadata	0x0080

struct VideoConfig
{
	int Width;
	int Height;
	double Framerate;
};

struct FrameConfig
{
	int Width;
	int Height;
	unsigned short BitCount;
	long long AvgTimePerFrame;
};

struct FrameMetadata
{
	unsigned short Size;	// size of this structure
	unsigned int Counter;	// frame counter
	unsigned int CounterHW;	// frame counter hardware
	long long Timestamp;	// time stamp in UNITS (1    per second)
	long long TimestampMedia;
	TFlagState FlagState;
	float TempChip;
	float TempFlag;
	float TempBox;
	WORD PIFin;
};

struct NewFrameBuffer
{
        void* data;
	long long *Timestamp;
	long long *TimestampMedia;
};

struct FrameBuffer
{
        void* data;
	PFrameMetadata metadata;
};

struct SnapshotBuffer
{
        void* EnergyBuffer;
        void* TempBuffer;
        void* IRColorBuffer;
        void* VisColorBuffer;
	FrameConfig EnergyConfig;
	FrameConfig TempConfig;
	FrameConfig IRColorConfig;
	FrameConfig VisColorConfig;
	FrameMetadata Metadata;
};


// Structures for format definition:
enum SubframeEntryType
{
	sfetSkip,
	sfetData,
	sfetMetadata
};

struct SubframeEntry
{
	SubframeEntryType type;
	int pixelcount;
};

struct SubframeBlock
{
	int offset;
	int count;
	int entrycount;
	SubframeEntry *entries;
};

struct SubframeDef
{
	int blockcount;
	SubframeBlock *blocks;
	int metadatasize;
	short *metadata_tmpbuffer;
	unsigned int *aDataIndex;
	unsigned int *aMetadataIndex;
};

struct FormatDef
{
        GUID guid;
	char *name;
	VideoConfig In;
	VideoConfig Out;
	int SubframesCount;
	SubframeDef *subframes;
};

enum TRefFitMode { RefFitMode_Auto, RefFitMode_Offset, RefFitMode_Gain };


struct TempCalcRefStruct
{
	TRefFitMode FitMode;
	float TRef;
	float TComp;
	short Count;
	short ConsiderFlag_Count; // Blocks displaying temps after Flag cycle;
};

struct TempCalcStruct
{
	bool ADUValues;
	float Emissivity;
	float Transmissivity;
	short AltEmissivitiesCount;
	float *AltEmissivities;
	float TempFlag;
	float TempAmb;
	bool UseReferencing;
	TempCalcRefStruct Ref;
};

struct PIFConfigStruct
{
	// PIF in
	TPifMode PifInMode;
	WORD PifInThreshold;  // Voltage in x * 10ms
	bool PifInLowActive;

	// PIF digital in
	TPifMode PifInDigitalMode;
	bool PifInDigitalLowActive;

	// PIF out
	TPifMode PifOutMode;
	WORD PifOutFlagOpen; // Voltage in x * 10ms
	WORD PifOutFlagClosed;
	WORD PifOutFlagMoving;
};

struct RecordingDetails
{
	FrameConfig IR;
	FrameConfig Vis;
	int IRVisRatio;
	int MetainfoLines;
};

typedef void *PVOID;
typedef PVOID HANDLE;
typedef HANDLE HINSTANCE;
typedef HINSTANCE HMODULE;
typedef unsigned long ULONG;
