#include <ros/ros.h>
#include <image_transport/image_transport.h>
#include <sensor_msgs/image_encodings.h>

#include "camera.h"
//#include "imager_filter.h"
#include <stdio.h>
#include "error.h"

#include <cmath>

#include "include/Imager.h"
#include "ImagerUVC.h"


using namespace optris;

image_transport::Publisher publisher;

class thermo_node{
    public:
    //ImagerUVC uvc;
    camera cam;
    Imager * imager;
    unsigned long serial;
    int Width;
    int Height;
    int count;

    public:

    unsigned char * buff;

    thermo_node():buff(0){
    };

    ~thermo_node(){
        if(buff)delete buff;
        delete imager;
    };
    void init(fptrOptrisFrame pOnFrame){
        /*
        serial = uvc.FindFirstDevice();
        if(serial==0){
            std::cout << "Error finding PI-IMAGER device" << std::endl;
            ros::shutdown();
            //exit(1);
        };
        if(uvc.OpenDevice()!=0){
            std::cout << "Error opening PI-IMAGER device: " << uvc.GetPath() << std::endl;
            ros::shutdown();
            //exit(1);
        };
        //imager_filter imager;
        Width = uvc.GetWidth();
        Height = uvc.GetHeight()-1;
        int frequency = uvc.GetFrequency();
        */
        cam.find_device();
        cam.init_device();
        Width = cam.width;
        Height = cam.height-1;
        serial = cam.dev_serial;
        int frequency = cam.frequency;

        imager = new Imager(serial, Width, Height+1, frequency, "../data/config/12050017.xml");
        imager->forceRecalibration();
        count = 0;
        //imager->setAutoflagControl(true);
//        uvc.Start();
        buff = new unsigned char[Width*(Height+1)*2];
        imager->setFrameCallback(pOnFrame);
    };
    void triger(){
        if(count==0){
            printf("thermo_node %4.4i  ",(count%10000)); fflush(stdout);
        };
        ++count;
        //uvc.GetFrame(buff);
		//imager->process(buff);
		//uvc.ReleaseFrame();
        cam.get_image();
        imager->process((unsigned char *)cam.buffer.start);
		//const char * psym = "-\\|/";
        //std::cout<<"\b"<<psym[count % 4];fflush(stdout);
        printf("\b\b\b\b%4.4i",(count%10000));fflush(stdout);
    };

    void cbOnFrame(unsigned short* image, unsigned int w, unsigned int h){
        //return;
		sensor_msgs::ImagePtr msg(new sensor_msgs::Image());
   		msg->header.stamp = ros::Time::now();
		int num_bytes = Width*Height*2;
		msg->data.resize(num_bytes);
		memcpy(&msg->data[0],image,num_bytes);
		msg->encoding = sensor_msgs::image_encodings::MONO16;
		msg->step = 2*Width;
		msg->height = Height;
		msg->width = Width;
		msg->header.frame_id = "/thermo_frame";
		msg->header.seq = count;
		publisher.publish(msg);
    };
};

thermo_node tcam;

void cbOnFrame(unsigned short* image, unsigned int w, unsigned int h){
    tcam.cbOnFrame(image,w,h);
};


int main(int argc, char **argv){
    ros::init(argc, argv, "thermo_node");
	ros::NodeHandle nh;
	image_transport::ImageTransport it(nh);
    publisher = it.advertise("thermo_camera/image", 1);

	tcam.init(cbOnFrame);
	int recali_time = 3; // recalibrate every 3 seconds
    int loop_rate = 30;
    ros::Rate r(loop_rate);
	int rcount = 0;
    while (ros::ok()){
		ros::spinOnce();
		++rcount;
		if(rcount>=recali_time*loop_rate){
            tcam.imager->forceRecalibration();
            rcount = 0;
		};
        tcam.triger();
        r.sleep();
        //usleep(1000);
    };
};
