#include <ros/ros.h>
#include <image_transport/image_transport.h>
#include <sensor_msgs/image_encodings.h>

#include "IronPalette.h"

#include <QtGui/QApplication>
#include <QtGui/QLabel>
#include <QtCore/QBasicTimer>
#include <QtGui/QPainter>
#include <QtGui/QDialog>
#include <QtGui/QKeyEvent>

#include <stdio.h>

class myApp : public QApplication {
public:
    QImage img;
    QLabel label;
    QLabel * ll[3][3];
    QLabel * info;
    int count;
public:
    myApp(int & argc, char **argv):QApplication(argc,argv), info(0){
        startTimer(1000/100);
    };

    ~myApp(){
        label.show();
        for(int i=0; i<3; ++i) {
            for(int j=0; j<3; ++j) {
                delete ll[i][j];
            };
        };
        delete info;
    };

    void init(int Width,int Height){
        label.resize(Width*4,Height*4);
        img = QImage(Width, Height, QImage::Format_RGB32);
        img.fill(QColor(0,0,0));
        label.setScaledContents(true);
        label.show();
        for(int i=0; i<3; ++i) {
            for(int j=0; j<3; ++j) {
                QLabel * l= new QLabel(&label);
                l->setText("36.6");
                l->move(Width*(j+1),Height*(i+1));
                l->show();
                ll[i][j] = l;
            };
        };
        ///
        info = new QLabel(&label);
        info->setText("Info");
        info->move(10,10);
        info->show();
        count = 0;
    };

    void imageCallback(const sensor_msgs::ImageConstPtr & msg) {
        //
        ++count;
        int Width = msg->width;
        int Height = msg->height;
        if(!info){
            init(Width,Height);
        };

        float * vals = new float[Width*Height];

        unsigned short * pV = (unsigned short*)&(msg->data[0]);

        for(int i = 0; i < (Width) * (Height); ++i) {
            vals[i] = ((float)(pV[i])-1000.0)*0.1;
        };

        QRgb * pimg = (QRgb*)img.bits();

        float max=-1e6;
        float min=+1e6;
        float * vals1 = new float[Width*Height];

        //median filter
        std::vector<float> a;
        a.resize(9);
        memset(vals1,0,Width*Height*sizeof(float));
        for(int i=1; i<Height-1; ++i) {
            for(int j=1; j<Width-1; ++j) {
                int ii = i*Width+j;
                a[0] = vals[ii];
                a[1] = vals[ii+1];
                a[2] = vals[ii-1];
                a[3] = vals[ii-Width];
                a[4] = vals[ii+Width];
                a[5] = vals[ii-1-Width];
                a[6] = vals[ii-1+Width];
                a[7] = vals[ii+1-Width];
                a[8] = vals[ii+1+Width];
                std::nth_element(a.begin(),a.begin()+5,a.end());
                float pixel = a[5];
                vals1[ii] = pixel;
                //vals1[ii] = vals[ii];
                if(pixel > max)max = pixel;
                if(pixel < min)min = pixel;
            };
        };

        // apply iron palete

        for(int i = 0; i < (Width) * (Height); ++i) {
            float pixel = vals1[i];
            int a = std::min(239,std::max(0,int((pixel-min)*240/(max-min+1))));
            pimg[i] = qRgb(Iron[a][0],Iron[a][1],Iron[a][2]);
        };

        char * s[1024];
//       temp_measure m;
        QPainter painter(&img);
        painter.setPen(QPen(Qt::white, 1));
        for(int i=0; i<4; ++i) {
            painter.drawLine(Width/8+Width/4*i,Height/8,Width/8+Width/4*i,Height*7/8);
            painter.drawLine(Width/8,Height/8+Height/4*i,Width*7/8,Height/8+Height/4*i);
        };
        for(int i=0; i<3; ++i) {
            for(int j=0; j<3; ++j) {
                float temp = 0;
                int n = 0;
                for(int i1 = -Height/8; i1<Height/8; ++i1) {
                    for(int j1 = -Width/8; j1<Width/8; ++j1) {
                        int jj = Width/4*(j+1)+j1;
                        int ii = (Height/4)*(i+1)+i1;
                        temp+= vals1[jj+ii*Width];
                        ++n;
                    };
                };
                temp/=n;
//                if(i==1 && j==1)m.energy = temp;
//                m.ee[i*3+j] = temp;
                sprintf((char*)s,"%3.1f",temp);
                ll[i][j]->setText(QString("<font color='white' size='8'>")+(char*)s+QString("</font>"));
                ll[i][j]->adjustSize();
            };
        };
        label.setPixmap(QPixmap::fromImage(img));
        //
        //sprintf((char*)s,"temperature chip: %3.1f, flag: %3.1f, box: %3.1f",imager.TempChip,imager.TempFlag, imager.TempBox);
        sprintf((char*)s,"%4.4i",count%10000);
        info->setText(QString("<font color='black' size='6'>")+(char*)s+QString("</font>"));
        info->adjustSize();

        delete[] vals1;
        delete[] vals;
        /*
        // display image
        img = QImage(Width,Height,QImage::Format_RGB32);
        label.resize(Width*4,Height*4);
        //
        QRgb * pimg = (QRgb*)img.bits();
        //uchar * pbuf = (uchar*)&msg->data[0];
        short int * vals = (short int*)&(msg->data[0]);
        int max=0;
        int min=1<<14;

        //int * vals = new int[Width * (Height)];

        for(int i = 0; i < (Width) * (Height); ++i){
            //vals[i] = pbuf[2*i+1] << 8 | pbuf[2*i];
            //vals[i] = pbuf[2*i+1] << 8 | pbuf[2*i];
            int pixel = vals[i];
        	if(pixel > max)max = pixel;
        	if(pixel < min)min = pixel;
        };
        //min = 2000;
        //max = 6000;
        min = 1200;
        max = 2000;
        for(int i = 0; i < (Width) * (Height); ++i){
        	int pixel = vals[i];
        	int a = std::min(239,std::max(0,(pixel-min)*240/(max-min+1)));
        	pimg[i] = qRgb(Iron[a][0],Iron[a][1],Iron[a][2]);
        };
        //delete vals;
        label.setPixmap(QPixmap::fromImage(img));
        	};
        	*/
    };
    //
    void timerEvent(QTimerEvent * event) {
        if(!ros::ok()){
            //ros::shutdown();
            exit(0);
        };
        ros::spinOnce();
    };

    bool notify( QObject * receiver, QEvent * ev) {
        if (ev->type() == QEvent::KeyPress) {
            QKeyEvent *e = static_cast<QKeyEvent *>(ev);
            //QMessageBox* box = new QMessageBox();
            //box->setWindowTitle(QString("Hello"));
            //box->setText(QString("You Pressed: ")+ e->text());
            //box->show();
            if(e->key() == Qt::Key_Space) {
                return true;
            } else if (e->key() == Qt::Key_Escape) {
                exit(0);
                return true;
            };
        };
        return QApplication::notify(receiver,ev);
    };
};

myApp *a;

void imageCallback(const sensor_msgs::ImageConstPtr & msg) {
    //fprintf(stderr,"!");
    a->imageCallback(msg);
};

int main(int argc, char **argv) {
    ros::init(argc, argv, "thermo_view");
    ros::NodeHandle nh;
    //
    image_transport::ImageTransport it1(nh);
    image_transport::Subscriber sub = it1.subscribe("thermo_camera/image", 1,imageCallback);
    //
    a = new myApp(argc, argv);
    a->exec();
    delete a;
};
